/*
 * Created on 19.06.2007
 * (c)2007 Lev Levin
 */
package ru.onego.industrial.wishlistupdater;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;

/**
 * Main class of the instrument.
 * @author Lev Levin
 * @version 1.0
 */
public class JWishListUpdater {
    
    // full path to the source 'wishlist.cfg'
    private String sourceFile;
    
    // full path to the destination text file
    private String destinationFile;
    
    /**
     * Loads internal properties.
     * @throws Exception Thrown 
     */
    private void loadProperties() throws Exception {
        Properties properties = new Properties();
        try {
            properties.load(new FileInputStream("updater.properties"));
        } catch (FileNotFoundException e) {
            System.out.println("'updater.properties' file is not found");
            throw new Exception();
        } catch (IOException e) {
            System.out.println("Could not load 'updater.properties'");
            throw new Exception();
        }
        sourceFile = properties.getProperty("source.file");
        if (sourceFile == null || sourceFile.trim().equals("")) {
            throw new Exception("source.file is not set in 'updater.properties'");
        }
        destinationFile = properties.getProperty("destination.file");
        if (destinationFile == null || destinationFile.trim().equals("")) {
            throw new Exception("destination.file is not set in 'updater.properties'");
        }
    }
    
    /**
     * Performs actual converting.
     * @throws Exception Thrown if any file error occurs
     */
    private void convert() throws Exception {
        System.out.println("- Performing converting -");
        // those streams are used to read/write data
        FileInputStream stream = new FileInputStream(sourceFile);
        FileWriter writer = new FileWriter(destinationFile);
        // loading first four bytes of the file
        // this header contains number of lines in this wishlist config file
        byte[] header = new byte[4];
        stream.read(header);
        System.out.println(String.format("Loading %s lines.", convertBytesToInt(header)));
        // next while end of stream is not reached, items are loaded in a loop
        int i = 0;
        while (stream.available() > 0) {
            // first four bytes of the record are length of string
            byte[] size = new byte[4];
            stream.read(size);
            byte[] str = new byte[convertBytesToInt(size)];
            // reading string itself
            stream.read(str);
            String string = new String(str);
            System.out.println("Loaded string: " + string);
            writer.write(string + System.getProperty("line.separator"));
            i++;
        }
        System.out.println("Loaded lines: " + i);
        // everything is done. closing streams
        stream.close();
        writer.close();
        System.out.println("- Convertion completed -");
    }
    
    /**
     * Converts array of bytes into integer value.
     * @param b Array of bytes
     * @return Integer value
     */
    protected int convertBytesToInt(byte[] b) {
        if (b == null || b.length != 4) {
            throw new IllegalArgumentException("Wrong data or data size");
        }
        return ((b[3] & 255) << 24) | ((b[2] & 255) << 16) | ((b[1] & 255) << 8) | ((b[0] & 255));
    }
    
    /**
     * Main method of the application.
     * @param args Program arguments. Not used in this version
     */
    public static void main(String[] args) {
        
        JWishListUpdater updater = new JWishListUpdater();
        
        System.out.println("JWishListUpdater 1.0.0.1");
        System.out.println("(c)2007 Lic");
        System.out.println();
        System.out.println("- Reading options -");
        try {
            updater.loadProperties();
            updater.convert();
        } catch (Exception e) {
            if (e.getMessage() != null) {
                System.out.println(e.getMessage());
            } else {
                System.out.println(e);
            }
            System.exit(1);
        }
        System.out.println("- Everything is done -");
        
    }

}
